import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:hub/backend/schema/enums/enums.dart';
import 'package:hub/components/atomic_components/menu_button_item/menu_button_item_widget.dart';
import 'package:hub/components/atomic_components/menu_card_item/menu_card_item.dart';
import 'package:hub/components/molecular_components/menu_item/menu_item.dart';
import 'package:hub/components/molecular_components/option_selection_modal/option_selection_modal_widget.dart';
import 'package:hub/components/organism_components/menu_component/menu_component_widget.dart';
import 'package:hub/components/organism_components/menu_list_view_component/menu_list_view_component_model.dart';
import 'package:hub/components/organism_components/menu_staggered_view_component/menu_staggered_view_component_model.dart';
import 'package:hub/flutter_flow/flutter_flow_util.dart';
import 'package:hub/flutter_flow/nav/nav.dart';
import 'package:hub/shared/extensions/dialog_extensions.dart';
import 'package:hub/shared/helpers/license/license_helper.dart';
import 'package:hub/shared/helpers/storage/base_storage.dart';
import 'package:hub/shared/helpers/storage/storage_helper.dart';
import 'package:hub/shared/services/authentication/authentication_service.dart';
import 'package:material_symbols_icons/symbols.dart';

import '/components/organism_components/menu_list_view_component/menu_list_view_component_widget.dart';
import 'menu_component_widget.dart' show MenuComponentWidget;

class MenuComponentModel extends FlutterFlowModel<MenuComponentWidget> {
  final MenuView style;
  final MenuItem item;
  final bool expandable;
  final List<Module> menuOptions;

  MenuComponentModel({required this.style, required this.item, required this.expandable, required this.menuOptions});

  bool isGrid = false;

  late MenuListViewComponentModel menuListViewComponentModel;
  late MenuStaggeredViewComponentModel menuStaggeredViewComponentModel;
  late VoidCallback safeSetState;
  late List<MenuEntry?> menuEntries;

  @override
  void initState(BuildContext context) {
    menuListViewComponentModel = createModel(context, () => MenuListViewComponentModel());
    menuStaggeredViewComponentModel = createModel(context, () => MenuStaggeredViewComponentModel());
  }

  @override
  void dispose() {
    menuListViewComponentModel.dispose();
    menuStaggeredViewComponentModel.dispose();
  }

  Future<List<MenuEntry?>> generateMenuEntries(BuildContext context, MenuItem item, List<Module> options) async {
    List<MenuEntry?> entries = [];
    Future<MenuEntry?> addMenuEntry(IconData icon, String enText, String ptText, Function() action) async {
      entries.add(
        item == MenuItem.button
            ? MenuButtonWidget(
                icon: icon,
                action: action,
                title: FFLocalizations.of(context).getVariableText(enText: enText, ptText: ptText),
                safeSetState: safeSetState)
            : item == MenuItem.card
                ? MenuCardItem(
                    icon: icon,
                    action: action,
                    title: FFLocalizations.of(context).getVariableText(enText: enText, ptText: ptText),
                    safeSetState: safeSetState)
                : item == MenuItem.tile
                    ? MenuCardItem(
                        icon: icon,
                        action: action,
                        title: FFLocalizations.of(context).getVariableText(enText: enText, ptText: ptText),
                        safeSetState: safeSetState)
                    : null,
      );
      return null;
    }

    for (var opt in options) {
      String? v = await LicenseHelper().g(opt.value);
      log('Module: ${opt.value} - License: $v');
      switch (v) {
        case 'VISIVEL':
          addMenuEntry(Icons.engineering_outlined, 'Schedule Providers', 'Agendar Prestadores', () async {
            await open(context, '/provisionalSchedule');
          });
          break;
        case 'INVISIVEL':
          DialogUnavailable.unavailableFeature(context);
          break;
        default:
          break;
      }
      safeSetState();
    }

    return entries;
  }

  Future open(BuildContext context, String link) async {
    context.push(
      link,
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future out(BuildContext context) async {
    final String title = FFLocalizations.of(context).getVariableText(
      ptText: 'Sair',
      enText: 'Logout',
    );
    final String content = FFLocalizations.of(context).getVariableText(
      ptText: 'Tem certeza que deseja sair?',
      enText: 'Are you sure you want to logout?',
    );
    showAlertDialog(context, title, content, () async => await AuthenticationService.signOut(context));
  }

  Future openDeliverySchedule(BuildContext context) async {
    final bool isProvisional = await StorageHelper().g(KeychainStorageKey.provisional.value) == 'true';
    if (isProvisional == true) {
      context.push(
        '/deliverySchedule',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future openProvisionalSchedule(BuildContext context) async {
    final isProvisional = await StorageHelper().g(KeychainStorageKey.provisional.value) == 'true';
    if (isProvisional == true) {
      context.push(
        '/provisionalSchedule',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future openFastPassSchedule(BuildContext context) async {
    final isWpp = await StorageHelper().g(KeychainStorageKey.whatsapp.value) == 'true';
    if (isWpp) {
      context.push(
        '/fastPassPage',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future showSchedules(BuildContext context) async {
    final routesListStr = <String>[
      'scheduleProvisionalVisitPage',
      'fastPassPage',
      'scheduleCompleteVisitPage',
    ];
    final iconsListIcon = <IconData>[
      Icons.date_range_rounded,
      Icons.date_range_rounded,
      Icons.date_range_rounded,
    ];
    final nameListStr = <String>[
      FFLocalizations.of(context).getVariableText(
        ptText: 'Visita\nProvisória',
        enText: 'Provisional\nSchedule',
      ),
      FFLocalizations.of(context).getVariableText(
        ptText: 'Visita\nRápida',
        enText: 'Fast\nSchedule',
      ),
      FFLocalizations.of(context).getVariableText(
        ptText: 'Visita\nCompleta',
        enText: 'Complete\nSchedule',
      ),
    ];
    await showAdaptiveDialog(
      context: context,
      builder: (context) {
        return Padding(
          padding: MediaQuery.viewInsetsOf(context),
          child: OptionSelectionModalWidget(
            routesListStr: routesListStr,
            iconsListIcon: iconsListIcon,
            nameListStr: nameListStr,
          ),
        );
      },
    );
  }

  Future openVisitorsRegister(BuildContext context) async {
    context.push(
      '/registerVisitorPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openPoepleOnTheProperty(BuildContext context) async {
    context.push(
      '/peopleOnThePropertyPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future<void> Logout(BuildContext context) async {
    final String title = FFLocalizations.of(context).getVariableText(
      ptText: 'Sair',
      enText: 'Logout',
    );
    final String content = FFLocalizations.of(context).getVariableText(
      ptText: 'Tem certeza que deseja sair?',
      enText: 'Are you sure you want to logout?',
    );
    showAlertDialog(context, title, content, () async => await AuthenticationService.signOut(context));
  }

  Future openPreferencesSettings(BuildContext context) async {
    context.push(
      '/preferencesSettings',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openMyOrders(BuildContext context) async {
    final isWpp = await StorageHelper().g(KeychainStorageKey.whatsapp.value) == 'true';
    ;
    if (isWpp) {
      context.push(
        '/packageOrder',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future openReservations(BuildContext context) async {
    final isWpp = await StorageHelper().g(KeychainStorageKey.whatsapp.value) == 'true';
    ;
    if (isWpp) {
      context.push(
        '/reservation',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future showHistories(BuildContext context) async {
    await showAdaptiveDialog(
      // isScrollControlled: true,
      // backgroundColor: Colors.transparent,
      // enableDrag: false,
      context: context,
      builder: (context) {
        return Padding(
          padding: MediaQuery.viewInsetsOf(context),
          child: OptionSelectionModalWidget(
            routesListStr: const <String>[
              'liberationHistory',
              'acessHistoryPage',
              'scheduleCompleteVisitPage',
              'messageHistoryPage'
            ],
            iconsListIcon: const <IconData>[
              Icons.history_rounded,
              Icons.history_rounded,
              Icons.history_rounded,
              Icons.history_rounded,
            ],
            nameListStr: <String>[
              FFLocalizations.of(context).getVariableText(
                ptText: 'Histórico\nde Liberação',
                enText: 'Liberation\nHistory',
              ),
              FFLocalizations.of(context).getVariableText(
                ptText: 'Histórico\nde Acesso',
                enText: 'Access\nHistory',
              ),
              FFLocalizations.of(context).getVariableText(
                ptText: 'Histórico\nde Visita',
                enText: 'Visit\nHistory',
              ),
              FFLocalizations.of(context).getVariableText(
                ptText: 'Histórico\nde Mensagens',
                enText: 'Message\nHistory',
              ),
            ],
          ),
        );
      },
    );
  }

  Future openLiberationsHistory(BuildContext context) async {
    context.push(
      '/liberationHistory',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openAccessHistory(BuildContext context) async {
    context.push(
      '/acessHistoryPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openVisitsHistory(BuildContext context) async {
    context.push(
      '/scheduleCompleteVisitPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openMessagesHistory(BuildContext context) async {
    context.push(
      '/messageHistoryPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openPetsRegister(BuildContext context) async {
    bool isPet = await StorageHelper().g(KeychainStorageKey.pets.value) == 'true';
    ;
    if (isPet) {
      context.push(
        '/petsPage',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }
}
